* zLib routines for FoxPro
* Requires L:\Tyler\Scripts\zlib.dll
*
* To use these functions in your program, add this line
*   set procedure to l:\tyler\scripts\zlib additive
* That will register the functions for the DLL for you and set up these three functions:
*   gzcompress(@dest, @destlen, source, sourcelen)
*   gzcompress2(@dest, @destlen, source, sourcelen, level)
*   gzuncompress(@dest, @destlen, source, sourcelen)
* See zlib.h (also in L:\tyler\scripts\) for information about these functions.
* See zconf.h (L:\tyler\scripts\) for information about the data types and return codes
*
* For use in your FoxPro program, you can try out these two utility functions:
*   compressed_data = compressit(data)  && default level (6)
*   compressed_data = compressit(data, level)  && level is a number from 0 to 9
*   uncompressed_data = decompressit(compressed_data)
*   gzip("in.dbf", "out.dbf.gz")  && default level (6)
*   gzip("in.dbf", "out.dbf.gz", level)  && level is a number from 0 to 9
*   gunzip("in.dbf.gz", "out.dbf")  && Shows a progress bar if talk is on
*   gunzip_nobar("in.dbf.gz", "out.dbf")  && Never shows a progress bar
* The compressed data is a string that has the size of the decompressed data,
* then a space, then the gzipped data.
*
* Sample program:
*    m = filetostr("your_file.txt")
*    n = compressit(m)  && Default compression (6)
*    n = compressit(m, 0)  && No compression (why bother?)
*    n = compressit(m, 9)  && Maximum compression
*    o = uncompressit(n)
*    strtofile(o, "the_same_data.txt")
*
*    * These functions return 0 on no error (if you want to check the return code)
*    gzip("file.dbf", "file.dbf.gz") && Default compression (6)
*    gzip("file.dbf", "file.dbf.gz", 0) && No compression (no use)
*    gzip("file.dbf", "file.dbf.gz", 9) && Maximum compression
*    gunzip("file.dbf.gz", "file.dbf")
*
* For updates, visit http://gnuwin32.sourceforge.net/packages/zlib.htm
* For more about zLib, visit http://www.gzip.org/zlib/



function GZDefine
	local m.zlib_dll
	public m.gz_block_size
	
	if sys(103) == "ON"
		set talk off
		m.zlib_dll = "L:\tyler\scripts\zlib.dll"
		m.gz_block_size = 65536
		set talk on
	else
		m.zlib_dll = "L:\tyler\scripts\zlib.dll"
		m.gz_block_size = 65536
	endif
	
	declare integer compress in (m.zlib_dll) ;
		as gzcompress string @ dest, long @ destlen, string src, long srclen
	declare integer compress2 in (m.zlib_dll) ;
		as gzcompress2 string @ dest, long @ destlen, string src, long srclen, integer lvl
	declare integer uncompress in (m.zlib_dll) ;
		as gzuncompress string @ dest, long @ destlen, string src, long srclen
	declare long gzopen in (m.zlib_dll) ;
		as gzopen string path, string mode
	declare integer gzread in (m.zlib_dll) ;
		as gzread long filegz, string @ data, integer length
	declare integer gzwrite in (m.zlib_dll) ;
		as gzwrite long filegz, string data, integer length
	declare integer gzeof in (m.zlib_dll) ;
		as gzeof long filegz
	declare integer gzclose in (m.zlib_dll) ;
		as gzclose long filegz
endfunc


* Returns a string of the compressed data with the size of the decompressed
* data prepended to it
Function CompressIt
parameter in_str, compress_lvl
	LOCAL DestSize, Dest, SourceSize, Res, m.talkSet
	
	if sys(103) == "ON"
		set talk off
		m.talkSet = "ON"
	else
		m.talkset = "OFF"
	endif
	
	SourceSize = len(in_str)
	DestSize = ceil(SourceSize * 1.001) + 12
	Dest = space(DestSize)
	if (! empty(compress_lvl))
		Res = gzcompress2(@Dest, @DestSize, in_str, SourceSize, compress_lvl)
	else
		Res = gzcompress(@Dest, @DestSize, in_str, SourceSize)
	endif
	set talk &talkSet
	
	If Res == 0 && Success
		return allt(str(SourceSize, 15)) + ' ' + left(Dest, DestSize)
	endif
	wait window "zlib compress:  Error code " + allt(str(Res))
	RETURN ''
endfunc


* Returns the decompressed data from a string that specifies the length of the
* decompressed data followed by a space and then the compressed data
Function DecompressIt
parameter in_str
	LOCAL Dest, DestSize, Res, m.talkset
	
	m.talkset = "SET TALK " + sys(103)
	set talk off
	Res = at(' ', in_str)
	DestSize = val(left(in_str, Res - 1))
	Dest = space(DestSize)
	in_str = substr(in_str, Res + 1)
	Res = gzuncompress(@Dest, @DestSize, in_str, len(in_str))
	&talkset
	
	if Res == 0 && Success
		return Dest
	endif
	wait window "zlib decompress:  Error code " + allt(str(Res))
	return ''
endfunc


function gzip_nobar
parameters m.srcfile, m.destfile
    local m.talkset, m.res
    
    if sys(103) == "ON"
    	set talk off
    	m.res = gzip(m.srcfile, m.destfile)
    	set talk on
    else
    	m.res = gzip(m.srcfile, m.destfile)
    endif
	
	return m.res
endfunc


function gzip
parameters m.srcfile, m.destfile, m.lvl
	local m.fp_in, m.fp_out, m.d, m.talkSet, m.BytesDone, m.NextUpdate, m.fileSize, m.l
	
	GZDefine()
	
	if sys(103) == "ON"
		set talk off
		m.talkSet = "ON"
		m.fileSize = fsize(m.srcfile)
	else
		m.talkSet = "OFF"
	endif

	m.fp_in = fopen(srcfile)
	if m.fp_in < 0
		wait window "Can not open " + m.srcfile + " for reading"
		SET TALK &talkSet
		return -1
	endif
	
	if (! empty(m.lvl))
		m.fp_out = gzopen(m.destfile, "wb" + allt(str(m.lvl)))
	else
		m.fp_out = gzopen(m.destfile, "wb")
	endif
	if m.fp_out == 0
		fclose(m.fp_in)
		wait window "Can not open " + m.destfile + " for writing"
		SET TALK &talkSet
		return -2
	endif
	
	IF m.talkSet="ON"
	        * Thermometer code
		* Part of the FoxPro Foundation Classes
		SET classLib TO F:\PUBLIC\FFC\_therm ADDITIVE

		m.BytesDone = 0
		m.NextUpdate = 0
		m.tTherm = CREATEOBJECT("_thermometer")
		m.tTherm.Caption = "Compressing File: " + m.srcfile
		m.tTherm.Show
		do while ! feof(m.fp_in)
			m.d = fread(m.fp_in, m.gz_block_size)
			m.l = len(m.d)
			gzwrite(m.fp_out, m.d, m.l)
			m.BytesDone = m.BytesDone + m.l
			if (m.NextUpdate <= m.BytesDone) then
				m.l = INT(100 * (m.BytesDone / m.fileSize))
				m.tTherm.iProgress = m.l
				m.tTherm.Update
				m.NextUpdate = (m.l + 1) / 100 * m.fileSize
			endif
		enddo
		m.tTherm.complete
		RELEASE m.tTherm
	ELSE
		do while ! feof(m.fp_in)
			m.d = fread(m.fp_in, m.gz_block_size)
			gzwrite(m.fp_out, m.d, len(m.d))
		enddo
	endif
	
	fclose(m.fp_in)
	gzclose(m.fp_out)

	SET TALK &talkSet
	return 0
endfunc


function gunzip_nobar
parameters m.srcfile, m.destfile
    local m.talkset, m.res
    
    if sys(103) == "ON"
    	set talk off
    	m.res = gunzip(m.srcfile, m.destfile)
    	set talk on
    else
    	m.res = gunzip(m.srcfile, m.destfile)
    endif
	
	return m.res
endfunc


function gunzip
parameters m.srcfile, m.destfile
	local m.fp_in, m.fp_out, m.d, m.siz, m.talkSet, m.tTherm, m.BytesDone, m.NextUpdate, m.i
	public m.gz_block_size
	
	GZDefine()

	if sys(103) == "ON"
		set talk off
		m.talkSet = "ON"
		m.fp_in = fopen(m.srcfile)
		fseek(m.fp_in, -4, 2)
		m.filesize = asc(fread(fp_in, 1))
		m.filesize = m.filesize + (asc(fread(m.fp_in, 1)) * 0xFF)
		m.filesize = m.filesize + (asc(fread(m.fp_in, 1)) * 0xFFFF)
		m.filesize = m.filesize + (asc(fread(m.fp_in, 1)) * 0xFFFFFF)
		fclose(m.fp_in)
	else
		m.talkSet = "OFF"
	ENDIF

	m.fp_in = gzopen(m.srcfile, "rb")
	if m.fp_in == 0
		wait window "Can not open " + m.srcfile + " for reading"
		set talk &talkSet
		return -1
	endif
	
	m.fp_out = fcreate(m.destfile)
	if m.fp_out < 0
		gzclose(m.fp_in)
		wait window "Can not open " + m.destfile + " for writing"
		set talk &talkSet
		return -2
	endif
	
	m.d = space(m.gz_block_size)
	
	IF m.talkSet="ON"
		SET classLib TO F:\PUBLIC\FFC\_therm ADDITIVE

		m.BytesDone = 0
		m.NextUpdate = 0
		m.tTherm = CREATEOBJECT("_thermometer")
		m.tTherm.Caption = "Uncompressing File: " + m.srcfile
		m.tTherm.Show
		do while gzeof(m.fp_in) == 0
			m.siz = gzread(m.fp_in, @m.d, m.gz_block_size)
			fwrite(m.fp_out, m.d, m.siz)
			m.BytesDone = m.BytesDone + m.siz
			if (m.NextUpdate <= m.BytesDone) then
				m.i = INT(100 * (m.BytesDone / m.fileSize))
				m.tTherm.iProgress = m.i
				m.tTherm.Update
				m.NextUpdate = (m.i + 1) / 100 * m.fileSize
			endif
		enddo
		m.tTherm.complete
		RELEASE m.tTherm
	ELSE
		do while gzeof(m.fp_in) == 0
			m.siz = gzread(m.fp_in, @m.d, m.gz_block_size)
			fwrite(m.fp_out, m.d, m.siz)
		enddo
	ENDIF
	
	gzclose(m.fp_in)
	fclose(m.fp_out)
	
	SET TALK &talkSet
	return 0
endfunc
